macroScript LightTable category:"Bobo_s Tools"
(

----------------------------------------------------------------
--Lighting Table
--Version 0.5
--Started : 2004/09/07
--Edited:   2008/12/17
--Code by Borislav Petrov  bobo@franticfilms.com
--Idea by John Redfern     
----------------------------------------------------------------

global LightTable_rollout
local theIniFile = getDir #plugcfg + "/lightTable.ini"

local layout_def = #()
local content_def = #()
local sort_alphabetically = false
local theVal = execute (getIniSetting theIniFile "Settings" "AlphaSort" )
if theVal != OK do sort_alphabetically = theVal

fn textsize txt = (gw.getTextExtent txt).x *1.4
	
fn compareFN v1 v2 valArray: =
(
	local v1i = valArray[v1][1].name
	local v2i = valArray[v2][1].name
	if v1i > v2i then 1 else -1
)


fn update_data =
(
	layout_def = #(#("Objects",100,undefined))
	content_def = #()

	for o in objects where findItem Light.classes (classof o) > 0 and  classof o != TargetObject do 
		append layout_def #(o.name,(textsize o.name),o)
	
	for o in objects where findItem geometryClass.classes (classof o ) > 0 and classof o != TargetObject do
	(
		content_line = #(o)
		for i = 2 to layout_def.count do
		(
			light = layout_def[i]
			if classof light[3] == Brazil_Light then
			(
				try(incList = BzGetListLight light[3] #illum mode:#exclude)catch(incList = undefined)
				try(exList = BzGetListLight light[3] #illum mode:#include)catch(exList = undefined)			
			)
			else
			(
				try(incList = light[3].includeList)catch(incList = undefined)
				try(exList = light[3].excludeList)catch(exList = undefined)
			)	
			testInclude = false		
			if incList != undefined then 
				testInclude = (findItem incList o > 0) 
			else 
				if exList != undefined then 
					testInclude = (findItem exList o == 0) 
				append content_line testInclude 
		)	
		append content_def content_line 
	)
	
	if sort_alphabetically then
	(
		local theIndexArray = for i = 1 to content_def.count collect i
		qsort theIndexArray compareFN valArray:content_def
		content_def = for i in theIndexArray collect content_def[i]
	)		
)
update_data()
	
fn initListView lv =
(
	lv.gridLines = true 
	lv.View = (dotNetClass "System.Windows.Forms.View").Details
	lv.fullRowSelect = true
	lv.checkboxes = false
	lv.multiSelect = false
	lv.Columns.Clear()
	lv.BackColor = lv.BackColor.FromArgb 230 235 245 
	for i in layout_def do
		lv.Columns.add i[1] i[2]
) 

fn fillInSpreadSheet lv =
(
	lv.Items.clear() --clear the list
	theRange = #() 
	for i in content_def do
	(
		li = dotNetObject "System.Windows.Forms.ListViewItem" i[1].name
		for j = 2 to i.count do
		(
			sub_li = li.SubItems.add (if i[j] then "X" else "--")
			sub_li.name = j as string
		)	
		append theRange li
	)
	lv.Items.AddRange theRange
) 

fn getSelection lv =
(
	for i in 1 to lv.Items.count do
	(
		li = lv.Items.item[i-1]
		if li.selected do return i
	)
	0
)

fn setSelection lv index =
(
	for i = 1 to lv.Items.count do
		lv.Items.item[i-1].selected = (i == index)
)

rcmenu LightTable_menu 
(
	menuItem LT_update "Update!"
	subMenu "Options..."
	(
		menuItem LT_alphasort "Sort Alphabetically" checked:sort_alphabetically
	)
	
	
	on LT_update picked do
	(
		update_data()
		initListView LightTable_rollout.lv_lights
		fillInSpreadSheet LightTable_rollout.lv_lights 	
	)
	
	on LT_alphasort picked do
	(
		sort_alphabetically = not sort_alphabetically
		setIniSetting theIniFile "Settings" "AlphaSort" (sort_alphabetically as string)
		update_data()
		initListView LightTable_rollout.lv_lights
		fillInSpreadSheet LightTable_rollout.lv_lights 			
	)

)

rollout LightTable_rollout "Lighting Table v0.5 (DotNet)"
(
	dotNetControl lv_lights "System.Windows.Forms.ListView" pos:[0,0]
	
	fn refresh_display =
	(
		update_data()
		initListView lv_lights
		fillInSpreadSheet lv_lights 	
	)
	
	fn selectClickedObject theSel =
	(
		theObject = content_def[theSel][1]
		if not keyboard.controlPressed then
		(
			if keyboard.altPressed do hide objects

			if isValidNode theObject then
			(					
				unhide theObject
				select theObject
			)	
			else
				refresh_display()
		)	
		else
		(
			if isValidNode theObject then
			(			
				selArray = #(theObject)
				for i = 2 to layout_def.count do
				(
					if content_def[theSel][i] and isValidNode layout_def[i][3] do
						append selArray layout_def[i][3]
				)
				if keyboard.altPressed do hide objects
	
				unhide selArray			
				select selArray
			)
			else
				refresh_display()			
		)	
	)
	
	on lv_lights ColumnClick header do
	(
		theIndex = header.Column + 1
		if theIndex == 1 then
			unhide objects
		else
		(
			theLight = layout_def[theIndex][3]
			if not keyboard.ControlPressed then
			(
				if keyboard.altPressed do hide objects
					
				if isValidNode theLight then
				(
					unhide theLight
					select theLight
					if theLight.target != undefined do 
					(
						unhide theLight.target
						select theLight.target
					)					
				)	
				else
					refresh_display()
			)
			else
			(
				if isValidNode theLight then
				(			
					selArray = #(theLight)
					if theLight.target != undefined do append selArray theLight.target
					for i in content_def do 
						if i[theIndex] and isValidNode i[1] do append selArray i[1]
					
					if keyboard.altPressed do hide objects	
					
					unhide selArray	
					select selArray	
				)	
				else
					refresh_display()			
			)	
		)
	)
	
	on lv_lights MouseClick theClick do
	(
		if theClick.Button == theClick.Button.Left then
		(
			local theItem = lv_lights.GetItemAt theClick.x theClick.y
			local theRowIndex = theItem.Index + 1
			
			local theSubItem = (theItem.GetSubItemAt theClick.x theClick.y)
			local theColIndex = ((theSubItem.name) as integer)-1
			local htest = [theRowIndex,theColIndex]
			
			if theColIndex < 1 then selectClickedObject theRowIndex
			
			if htest[1] > 0 and htest[2] > 0 then
			(
				local theLight = layout_def[htest[2]+1][3]
				local theObject = content_def[htest[1]][1]
				if isValidNode theLight and isValidNode theObject then
				(
					select #(theLight,theObject)
					theInclusion = content_def[htest[1]][htest[2]+1]
					theInclusion = not theInclusion 
					content_def[htest[1]][htest[2]+1] = theInclusion 
					
					if classof theLight == Brazil_Light then
					(
						try(incList = BzGetListLight theLight #illum mode:#exclude)catch(incList = undefined)
						try(exList = BzGetListLight theLight #illum mode:#include)catch(exList = undefined)
					)
					else				
					(
						try(incList = theLight.includeList)catch(incList = undefined)
						try(exList = theLight.excludeList)catch(exList = undefined)
					)	
					
					if theInclusion then
					(
						if incList != undefined then
						(
							append incList theObject
							if classof theLight == Brazil_Light then
								BzSetListLight theLight #illum incList mode:#exclude
							else
								theLight.includeList = incList 
						)	
						else
						(
							if exList != undefined do
							(
								theItem = findItem exList theObject
								if theItem > 0 do
								(
									deleteItem exList theItem
									if classof theLight == Brazil_Light then
										BzSetListLight theLight #illum exList mode:#include
									else
										theLight.excludeList = exList 
								)	
							)	
						)	
					)
					else
					(
						if exList != undefined then
						(
							append exList theObject
							if classof theLight == Brazil_Light then
								BzSetListLight theLight #illum exList mode:#include
							else
								theLight.excludeList = exList 
						)	
						else
						(
							if incList != undefined do
							(
								theItem = findItem incList theObject
								if theItem > 0 do
								(
									deleteItem incList theItem
									if classof theLight == Brazil_Light then
										BzSetListLight theLight #illum incList mode:#exclude
									else								
										theLight.includeList = incList 
								)	
							)	
						)	
					)	
					fillInSpreadSheet lv_lights 
					setSelection lv_lights -1
				)
			)	
		)	
	)
	
	on LightTable_rollout open do
	(
		initListView lv_lights
		fillInSpreadSheet lv_lights 
	)
	
	on LightTable_rollout moved pos do
	(
		setIniSetting TheIniFile "Dialog" "Position" (pos as string)
	)
	
	on LightTable_rollout resized new_size do 
	(
		lv_lights.height = new_size[2]
		lv_lights.width = new_size[1]
		setIniSetting TheIniFile "Dialog" "Size" (new_size as string)
	)	

)
thePos = getIniSetting TheIniFile "Dialog" "Position" 
if thePos != "" then thePos = execute thePos else thePos = [100,100]
theSize = getIniSetting TheIniFile "Dialog" "Size" 
if theSize!= "" then theSize = execute theSize else theSize= [400,300]


try(destroyDialog LightTable_rollout)catch()
createDialog LightTable_rollout theSize.x theSize.y thePos.x thePos.y style:#(#style_resizing, #style_titlebar, #style_sysmenu, #style_minimizebox, #style_maximizebox) menu:LightTable_menu 

LightTable_rollout.lv_lights.height = theSize[2]
LightTable_rollout.lv_lights.width= theSize[1]
)